@@ -1,10 +1,11 @@
-from django.conf.urls import url
+from django.urls import path
 
 from wagtail.api.v2.router import WagtailAPIRouter
 from wagtail.core import hooks
 
 from .views import PagesAdminAPIViewSet
 
+
 admin_api = WagtailAPIRouter('wagtailadmin_api')
 admin_api.register_endpoint('pages', PagesAdminAPIViewSet)
 
@@ -12,5 +13,5 @@ for fn in hooks.get_hooks('construct_admin_api'):
     fn(admin_api)
 
 urlpatterns = [
-    url(r'^main/', admin_api.urls),
+    path('main/', admin_api.urls),
 ]
