@@ -1,5 +1,4 @@
 import json
-import warnings
 
 from datetime import datetime
 from urllib.parse import urljoin
@@ -21,20 +20,19 @@ from django.utils.timesince import timesince
 from django.utils.translation import gettext_lazy as _
 
 from wagtail.admin.localization import get_js_translation_strings
-from wagtail.admin.log_action_registry import registry as log_action_registry
 from wagtail.admin.menu import admin_menu
 from wagtail.admin.navigation import get_explorable_root_page
 from wagtail.admin.search import admin_search_areas
 from wagtail.admin.staticfiles import versioned_static as versioned_static_func
 from wagtail.core import hooks
+from wagtail.core.log_actions import page_log_action_registry
 from wagtail.core.models import (
     Collection, CollectionViewRestriction, Locale, Page, PageLogEntry, PageViewRestriction,
     UserPagePermissionsProxy)
-from wagtail.core.utils import accepts_kwarg, camelcase_to_underscore
+from wagtail.core.utils import camelcase_to_underscore
 from wagtail.core.utils import cautious_slugify as _cautious_slugify
 from wagtail.core.utils import escape_script
 from wagtail.users.utils import get_gravatar_url
-from wagtail.utils.deprecation import RemovedInWagtail212Warning
 
 
 register = template.Library()
@@ -67,7 +65,7 @@ def main_nav(context):
 
 
 @register.inclusion_tag('wagtailadmin/shared/breadcrumb.html', takes_context=True)
-def explorer_breadcrumb(context, page, include_self=False):
+def explorer_breadcrumb(context, page, include_self=True, trailing_arrow=False):
     user = context['request'].user
 
     # find the closest common ancestor of the pages that this user has direct explore permission
@@ -77,7 +75,8 @@ def explorer_breadcrumb(context, page, include_self=False):
         return {'pages': Page.objects.none()}
 
     return {
-        'pages': page.get_ancestors(inclusive=include_self).descendant_of(cca, inclusive=True).specific()
+        'pages': page.get_ancestors(inclusive=include_self).descendant_of(cca, inclusive=True).specific(),
+        'trailing_arrow': trailing_arrow
     }
 
 
@@ -442,15 +441,7 @@ def page_listing_buttons(context, page, page_perms, is_parent=False):
 
     buttons = []
     for hook in button_hooks:
-        if accepts_kwarg(hook, 'next_url'):
-            buttons.extend(hook(page, page_perms, is_parent, next_url))
-        else:
-            warnings.warn(
-                'register_page_listing_buttons hooks will require an additional kwarg `next_url` in a future release. '
-                'Please update your hook function to accept `next_url`.',
-                RemovedInWagtail212Warning
-            )
-            buttons.extend(hook(page, page_perms, is_parent))
+        buttons.extend(hook(page, page_perms, is_parent, next_url))
 
     buttons.sort()
 
@@ -484,14 +475,14 @@ def admin_urlquote(value):
 
 
 @register.simple_tag
-def avatar_url(user, size=50):
+def avatar_url(user, size=50, gravatar_only=False):
     """
     A template tag that receives a user and size and return
     the appropiate avatar url for that user.
     Example usage: {% avatar_url request.user 50 %}
     """
 
-    if hasattr(user, 'wagtail_userprofile') and user.wagtail_userprofile.avatar:
+    if not gravatar_only and hasattr(user, 'wagtail_userprofile') and user.wagtail_userprofile.avatar:
         return user.wagtail_userprofile.avatar.url
 
     if hasattr(user, 'email'):
@@ -592,7 +583,7 @@ def timesince_last_update(last_update, time_prefix='', use_shorthand=True):
 def format_action_log_message(log_entry):
     if not isinstance(log_entry, PageLogEntry):
         return ''
-    return log_action_registry.format_message(log_entry)
+    return page_log_action_registry.format_message(log_entry)
 
 
 @register.simple_tag
@@ -654,3 +645,8 @@ def locales():
         }
         for locale in Locale.objects.all()
     ])
+
+
+@register.simple_tag
+def get_comments_enabled():
+    return getattr(settings, 'WAGTAILADMIN_COMMENTS_ENABLED', True)
