@@ -1,10 +1,10 @@
 from django.core.paginator import Paginator
 from django.http import Http404
-from django.shortcuts import get_object_or_404, render
+from django.shortcuts import get_object_or_404
+from django.template.response import TemplateResponse
 
 from wagtail.admin.forms.choosers import (
     AnchorLinkChooserForm, EmailLinkChooserForm, ExternalLinkChooserForm, PhoneLinkChooserForm)
-
 from wagtail.admin.forms.search import SearchForm
 from wagtail.admin.modal_workflow import render_modal_workflow
 from wagtail.core import hooks
@@ -43,15 +43,6 @@ def page_models_from_string(string):
     return tuple(page_models)
 
 
-def filter_page_type(queryset, page_models):
-    qs = queryset.none()
-
-    for model in page_models:
-        qs |= queryset.type(model)
-
-    return qs
-
-
 def can_choose_page(page, permission_proxy, desired_classes, can_choose_root=True, user_perm=None):
     """Returns boolean indicating of the user can choose page.
     will check if the root page can be selected and if user permissions
@@ -88,13 +79,13 @@ def browse(request, parent_page_id=None):
         # Find the highest common ancestor for the specific classes passed in
         # In many cases, such as selecting an EventPage under an EventIndex,
         # this will help the administrator find their page quicker.
-        all_desired_pages = filter_page_type(Page.objects.all(), desired_classes)
+        all_desired_pages = Page.objects.all().type(*desired_classes)
         parent_page = all_desired_pages.first_common_ancestor()
 
     parent_page = parent_page.specific
 
-    # Get children of parent page
-    pages = parent_page.get_children().specific()
+    # Get children of parent page (without streamfields)
+    pages = parent_page.get_children().defer_streamfields().specific()
 
     # allow hooks to modify the queryset
     for hook in hooks.get_hooks('construct_page_chooser_queryset'):
@@ -105,7 +96,7 @@ def browse(request, parent_page_id=None):
         # restrict the page listing to just those pages that:
         # - are of the given content type (taking into account class inheritance)
         # - or can be navigated into (i.e. have children)
-        choosable_pages = filter_page_type(pages, desired_classes)
+        choosable_pages = pages.type(*desired_classes)
         descendable_pages = pages.filter(numchild__gt=0)
         pages = choosable_pages | descendable_pages
 
@@ -167,7 +158,7 @@ def search(request, parent_page_id=None):
         pages = pages.exclude(
             depth=1  # never include root
         )
-        pages = filter_page_type(pages, desired_classes)
+        pages = pages.type(*desired_classes)
         pages = pages.specific()
         pages = pages.search(search_form.cleaned_data['q'])
     else:
@@ -179,7 +170,7 @@ def search(request, parent_page_id=None):
     for page in pages:
         page.can_choose = True
 
-    return render(
+    return TemplateResponse(
         request, 'wagtailadmin/chooser/_search_results.html',
         shared_context(request, {
             'searchform': search_form,
