@@ -1,13 +1,13 @@
 from django.contrib.auth.models import Permission
 from django.urls import reverse
 from django.utils.http import urlencode
-from django.utils.translation import gettext_lazy as _
 from django.utils.translation import gettext
+from django.utils.translation import gettext_lazy as _
 from draftjs_exporter.dom import DOM
 
 import wagtail.admin.rich_text.editors.draftail.features as draftail_features
+
 from wagtail.admin.auth import user_has_any_page_permission
-from wagtail.admin.localization import get_available_admin_languages, get_available_admin_time_zones
 from wagtail.admin.menu import MenuItem, SubmenuMenuItem, reports_menu, settings_menu
 from wagtail.admin.navigation import get_explorable_root_page
 from wagtail.admin.rich_text import (
@@ -20,7 +20,6 @@ from wagtail.admin.rich_text.converters.html_to_contentstate import (
     InlineStyleElementHandler, ListElementHandler, ListItemElementHandler, PageLinkElementHandler)
 from wagtail.admin.search import SearchArea
 from wagtail.admin.site_summary import PagesSummaryItem
-from wagtail.admin.views.account import email_management_enabled, password_management_enabled
 from wagtail.admin.viewsets import viewsets
 from wagtail.admin.widgets import Button, ButtonWithDropdownFromHook, PageListingButton
 from wagtail.core import hooks
@@ -78,7 +77,7 @@ class PageSearchArea(SearchArea):
         super().__init__(
             _('Pages'), reverse('wagtailadmin_pages:search'),
             name='pages',
-            classnames='icon icon-folder-open-inverse',
+            icon_name='folder-open-inverse',
             order=100)
 
     def is_shown(self, request):
@@ -203,7 +202,6 @@ def page_listing_more_buttons(page, page_perms, is_parent=False, next_url=None):
         if next_url:
             url += '?' + urlencode({'next': next_url})
 
-        urlencode
         yield Button(
             _('Copy'),
             url,
@@ -248,75 +246,6 @@ def register_viewsets_urls():
     return viewsets.get_urlpatterns()
 
 
-@hooks.register('register_account_menu_item')
-def register_account_set_profile_picture(request):
-    return {
-        'url': reverse('wagtailadmin_account_change_avatar'),
-        'label': _('Set profile picture'),
-        'help_text': _("Change your profile picture.")
-    }
-
-
-@hooks.register('register_account_menu_item')
-def register_account_change_email(request):
-    if email_management_enabled():
-        return {
-            'url': reverse('wagtailadmin_account_change_email'),
-            'label': _('Change email'),
-            'help_text': _('Change the email address linked to your account.'),
-        }
-
-
-@hooks.register('register_account_menu_item')
-def register_account_change_password(request):
-    if password_management_enabled() and request.user.has_usable_password():
-        return {
-            'url': reverse('wagtailadmin_account_change_password'),
-            'label': _('Change password'),
-            'help_text': _('Change the password you use to log in.'),
-        }
-
-
-@hooks.register('register_account_menu_item')
-def register_account_notification_preferences(request):
-    user_perms = UserPagePermissionsProxy(request.user)
-    if user_perms.can_edit_pages() or user_perms.can_publish_pages():
-        return {
-            'url': reverse('wagtailadmin_account_notification_preferences'),
-            'label': _('Notification preferences'),
-            'help_text': _('Choose which email notifications to receive.'),
-        }
-
-
-@hooks.register('register_account_menu_item')
-def register_account_preferred_language_preferences(request):
-    if len(get_available_admin_languages()) > 1:
-        return {
-            'url': reverse('wagtailadmin_account_language_preferences'),
-            'label': _('Language preferences'),
-            'help_text': _('Choose the language you want to use here.'),
-        }
-
-
-@hooks.register('register_account_menu_item')
-def register_account_current_time_zone(request):
-    if len(get_available_admin_time_zones()) > 1:
-        return {
-            'url': reverse('wagtailadmin_account_current_time_zone'),
-            'label': _('Current Time Zone'),
-            'help_text': _('Choose your current time zone.'),
-        }
-
-
-@hooks.register('register_account_menu_item')
-def register_account_change_name(request):
-    return {
-        'url': reverse('wagtailadmin_account_change_name'),
-        'label': _('Change name'),
-        'help_text': _('Change your first and last name on your account.'),
-    }
-
-
 @hooks.register('register_rich_text_features')
 def register_core_features(features):
     # Hallo.js
@@ -718,12 +647,20 @@ def register_icons(icons):
         'bin.svg',
         'bold.svg',
         'chain-broken.svg',
+        'check.svg',
+        'chevron-down.svg',
         'clipboard-list.svg',
         'code.svg',
         'cog.svg',
         'cogs.svg',
         'collapse-down.svg',
         'collapse-up.svg',
+        'comment.svg',
+        'comment-add.svg',
+        'comment-add-reversed.svg',
+        'comment-large.svg',
+        'comment-large-outline.svg',
+        'comment-large-reversed.svg',
         'cross.svg',
         'date.svg',
         'doc-empty-inverse.svg',
@@ -735,6 +672,7 @@ def register_icons(icons):
         'draft.svg',
         'duplicate.svg',
         'edit.svg',
+        'ellipsis-v.svg',
         'error.svg',
         'folder-inverse.svg',
         'folder-open-1.svg',
@@ -748,6 +686,7 @@ def register_icons(icons):
         'home.svg',
         'horizontalrule.svg',
         'image.svg',  # aka picture
+        'info-circle.svg',
         'italic.svg',
         'link.svg',
         'link-external.svg',
@@ -808,175 +747,3 @@ def register_icons(icons):
 @hooks.register('construct_homepage_summary_items')
 def add_pages_summary_item(request, items):
     items.insert(0, PagesSummaryItem(request))
-
-
-@hooks.register('register_log_actions')
-def register_core_log_actions(actions):
-    actions.register_action('wagtail.create', _('Create'), _('Created'))
-    actions.register_action('wagtail.edit', _('Save draft'), _('Draft saved'))
-    actions.register_action('wagtail.delete', _('Delete'), _('Deleted'))
-    actions.register_action('wagtail.publish', _('Publish'), _('Published'))
-    actions.register_action('wagtail.publish.scheduled', _("Publish scheduled draft"), _('Published scheduled draft'))
-    actions.register_action('wagtail.unpublish', _('Unpublish'), _('Unpublished'))
-    actions.register_action('wagtail.unpublish.scheduled', _('Unpublish scheduled draft'), _('Unpublished scheduled draft'))
-    actions.register_action('wagtail.lock', _('Lock'), _('Locked'))
-    actions.register_action('wagtail.unlock', _('Unlock'), _('Unlocked'))
-    actions.register_action('wagtail.moderation.approve', _('Approve'), _('Approved'))
-    actions.register_action('wagtail.moderation.reject', _('Reject'), _('Rejected'))
-
-    def revert_message(data):
-        try:
-            return _('Reverted to previous revision with id %(revision_id)s from %(created_at)s') % {
-                'revision_id': data['revision']['id'],
-                'created_at': data['revision']['created'],
-            }
-        except KeyError:
-            return _('Reverted to previous revision')
-
-    def copy_message(data):
-        try:
-            return _('Copied from %(title)s') % {
-                'title': data['source']['title'],
-            }
-        except KeyError:
-            return _("Copied")
-
-    def move_message(data):
-        try:
-            return _("Moved from '%(old_parent)s' to '%(new_parent)s'") % {
-                'old_parent': data['source']['title'],
-                'new_parent': data['destination']['title'],
-            }
-        except KeyError:
-            return _('Moved')
-
-    def schedule_publish_message(data):
-        try:
-            if data['revision']['has_live_version']:
-                return _('Revision %(revision_id)s from %(created_at)s scheduled for publishing at %(go_live_at)s.') % {
-                    'revision_id': data['revision']['id'],
-                    'created_at': data['revision']['created'],
-                    'go_live_at': data['revision']['go_live_at'],
-                }
-            else:
-                return _('Page scheduled for publishing at %(go_live_at)s') % {
-                    'go_live_at': data['revision']['go_live_at'],
-                }
-        except KeyError:
-            return _('Page scheduled for publishing')
-
-    def unschedule_publish_message(data):
-        try:
-            if data['revision']['has_live_version']:
-                return _('Revision %(revision_id)s from %(created_at)s unscheduled from publishing at %(go_live_at)s.') % {
-                    'revision_id': data['revision']['id'],
-                    'created_at': data['revision']['created'],
-                    'go_live_at': data['revision']['go_live_at'],
-                }
-            else:
-                return _('Page unscheduled for publishing at %(go_live_at)s') % {
-                    'go_live_at': data['revision']['go_live_at'],
-                }
-        except KeyError:
-            return _('Page unscheduled from publishing')
-
-    def add_view_restriction(data):
-        try:
-            return _("Added the '%(restriction)s' view restriction") % {
-                'restriction': data['restriction']['title'],
-            }
-        except KeyError:
-            return _('Added view restriction')
-
-    def edit_view_restriction(data):
-        try:
-            return _("Updated the view restriction to '%(restriction)s'") % {
-                'restriction': data['restriction']['title'],
-            }
-        except KeyError:
-            return _('Updated view restriction')
-
-    def delete_view_restriction(data):
-        try:
-            return _("Removed the '%(restriction)s' view restriction") % {
-                'restriction': data['restriction']['title'],
-            }
-        except KeyError:
-            return _('Removed view restriction')
-
-    def rename_message(data):
-        try:
-            return _("Renamed from '%(old)s' to '%(new)s'") % {
-                'old': data['title']['old'],
-                'new': data['title']['new'],
-            }
-        except KeyError:
-            return _('Renamed')
-
-    actions.register_action('wagtail.rename', _('Rename'), rename_message)
-    actions.register_action('wagtail.revert', _('Revert'), revert_message)
-    actions.register_action('wagtail.copy', _('Copy'), copy_message)
-    actions.register_action('wagtail.move', _('Move'), move_message)
-    actions.register_action('wagtail.publish.schedule', _("Schedule publication"), schedule_publish_message)
-    actions.register_action('wagtail.schedule.cancel', _("Unschedule publication"), unschedule_publish_message)
-    actions.register_action('wagtail.view_restriction.create', _("Add view restrictions"), add_view_restriction)
-    actions.register_action('wagtail.view_restriction.edit', _("Update view restrictions"), edit_view_restriction)
-    actions.register_action('wagtail.view_restriction.delete', _("Remove view restrictions"), delete_view_restriction)
-
-
-@hooks.register('register_log_actions')
-def register_workflow_log_actions(actions):
-    def workflow_start_message(data):
-        try:
-            return _("'%(workflow)s' started. Next step '%(task)s'") % {
-                'workflow': data['workflow']['title'],
-                'task': data['workflow']['next']['title'],
-            }
-        except (KeyError, TypeError):
-            return _('Workflow started')
-
-    def workflow_approve_message(data):
-        try:
-            if data['workflow']['next']:
-                return _("Approved at '%(task)s'. Next step '%(next_task)s'") % {
-                    'task': data['workflow']['task']['title'],
-                    'next_task': data['workflow']['next']['title'],
-                }
-            else:
-                return _("Approved at '%(task)s'. '%(workflow)s' complete") % {
-                    'task': data['workflow']['task']['title'],
-                    'workflow': data['workflow']['title'],
-                }
-        except (KeyError, TypeError):
-            return _('Workflow task approved')
-
-    def workflow_reject_message(data):
-        try:
-            return _("Rejected at '%(task)s'. Changes requested") % {
-                'task': data['workflow']['task']['title'],
-            }
-        except (KeyError, TypeError):
-            return _('Workflow task rejected. Workflow complete')
-
-    def workflow_resume_message(data):
-        try:
-            return _("Resubmitted '%(task)s'. Workflow resumed'") % {
-                'task': data['workflow']['task']['title'],
-            }
-        except (KeyError, TypeError):
-            return _('Workflow task resubmitted. Workflow resumed')
-
-    def workflow_cancel_message(data):
-        try:
-            return _("Cancelled '%(workflow)s' at '%(task)s'") % {
-                'workflow': data['workflow']['title'],
-                'task': data['workflow']['task']['title'],
-            }
-        except (KeyError, TypeError):
-            return _('Workflow cancelled')
-
-    actions.register_action('wagtail.workflow.start', _('Workflow: start'), workflow_start_message)
-    actions.register_action('wagtail.workflow.approve', _('Workflow: approve task'), workflow_approve_message)
-    actions.register_action('wagtail.workflow.reject', _('Workflow: reject task'), workflow_reject_message)
-    actions.register_action('wagtail.workflow.resume', _('Workflow: resume task'), workflow_resume_message)
-    actions.register_action('wagtail.workflow.cancel', _('Workflow: cancel'), workflow_cancel_message)
