@@ -1,6 +1,6 @@
 from django import forms
 from django.utils.safestring import mark_safe
-from django.utils.translation import ugettext as _
+from django.utils.translation import gettext as _
 
 from wagtail.core.models import Page
 
@@ -8,7 +8,7 @@ from wagtail.core.models import Page
 class PageChoiceField(forms.ModelChoiceField):
     def label_from_instance(self, obj):
         bits = []
-        for ancestor in obj.get_ancestors(inclusive=True).exclude(depth=1):
+        for ancestor in obj.get_ancestors(inclusive=True).exclude(depth=1).specific(defer=True):
             bits.append(ancestor.get_admin_display_title())
         return mark_safe('<span class="icon icon-arrow-right"></span>'.join(bits))
 
