@@ -2,42 +2,16 @@ from django.apps import apps
 from django.contrib.contenttypes.fields import GenericForeignKey, GenericRelation
 from django.contrib.contenttypes.models import ContentType
 from django.contrib.postgres.indexes import GinIndex
-from django.contrib.postgres.search import SearchQuery, SearchVectorField
-from django.db.models import CASCADE, ForeignKey, Model, TextField
+from django.contrib.postgres.search import SearchVectorField
+from django.db import models
 from django.db.models.functions import Cast
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import gettext_lazy as _
 
 from wagtail.search.index import class_is_indexed
 
 from .utils import get_descendants_content_types_pks
 
 
-class RawSearchQuery(SearchQuery):
-    def __init__(self, format, *args, **kwargs):
-        self.format = format
-        super().__init__(*args, **kwargs)
-
-    def as_sql(self, compiler, connection):
-        # escape apostrophe and backslash
-        params = [v.replace("'", "''").replace("\\", "\\\\") for v in self.value]
-        if self.config:
-            config_sql, config_params = compiler.compile(self.config)
-            template = "to_tsquery(%s::regconfig, '%s')" % (config_sql, self.format)
-            params = config_params + params
-        else:
-            template = "to_tsquery('%s')" % self.format
-        if self.invert:
-            template = '!!({})'.format(template)
-        return template, params
-
-    def __invert__(self):
-        extra = {
-            'invert': not self.invert,
-            'config': self.config,
-        }
-        return type(self)(self.format, self.value, **extra)
-
-
 class TextIDGenericRelation(GenericRelation):
     auto_created = True
 
@@ -66,21 +40,30 @@ class TextIDGenericRelation(GenericRelation):
         return []
 
 
-class IndexEntry(Model):
-    content_type = ForeignKey(ContentType, on_delete=CASCADE)
+class IndexEntry(models.Model):
+    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
     # We do not use an IntegerField since primary keys are not always integers.
-    object_id = TextField()
+    object_id = models.TextField()
     content_object = GenericForeignKey()
 
     # TODO: Add per-object boosting.
     autocomplete = SearchVectorField()
+    title = SearchVectorField()
+    # This field stores the "Title Normalisation Factor"
+    # This factor is multiplied onto the the rank of the title field.
+    # This allows us to apply a boost to results with shorter titles
+    # elevating more specific matches to the top.
+    title_norm = models.FloatField(default=1.0)
     body = SearchVectorField()
 
     class Meta:
         unique_together = ('content_type', 'object_id')
         verbose_name = _('index entry')
         verbose_name_plural = _('index entries')
+        # An additional computed GIN index on 'title || body' is created in a SQL migration
+        # covers the default case of PostgresSearchQueryCompiler.get_index_vectors.
         indexes = [GinIndex(fields=['autocomplete']),
+                   GinIndex(fields=['title']),
                    GinIndex(fields=['body'])]
 
     def __str__(self):
