@@ -1,6 +1,12 @@
 from django.contrib.sitemaps import Sitemap as DjangoSitemap
 
 
+# Note: avoid importing models here. This module is imported from __init__.py
+# which causes it to be loaded early in startup if wagtail.contrib.sitemaps is
+# included in INSTALLED_APPS (not required, but developers are likely to add it
+# anyhow) leading to an AppRegistryNotReady exception.
+
+
 class Sitemap(DjangoSitemap):
 
     def __init__(self, request=None):
@@ -15,9 +21,9 @@ class Sitemap(DjangoSitemap):
         return (obj.last_published_at or obj.latest_revision_created_at)
 
     def get_wagtail_site(self):
-        site = getattr(self.request, 'site', None)
+        from wagtail.core.models import Site
+        site = Site.find_for_request(self.request)
         if site is None:
-            from wagtail.core.models import Site
             return Site.objects.select_related(
                 'root_page'
             ).get(is_default_site=True)
@@ -31,7 +37,9 @@ class Sitemap(DjangoSitemap):
             .live()
             .public()
             .order_by('path')
-            .specific())
+            .defer_streamfields()
+            .specific()
+        )
 
     def _urls(self, page, protocol, domain):
         urls = []
