@@ -1,11 +1,13 @@
 import posixpath
+import warnings
+
 from collections import defaultdict
 
 from django.apps import apps
 from django.contrib.contenttypes.models import ContentType
 from django.db.models import CharField, Q
 from django.db.models.functions import Length, Substr
-from django.db.models.query import BaseIterable
+from django.db.models.query import BaseIterable, ModelIterable
 from treebeard.mp_tree import MP_NodeQuerySet
 
 from wagtail.search.queryset import SearchableQuerySetMixin
@@ -15,6 +17,12 @@ class TreeQuerySet(MP_NodeQuerySet):
     """
     Extends Treebeard's MP_NodeQuerySet with additional useful tree-related operations.
     """
+    def delete(self):
+        """Redefine the delete method unbound, so we can set the queryset_only parameter. """
+        super().delete()
+
+    delete.queryset_only = True
+
     def descendant_of_q(self, other, inclusive=False):
         q = Q(path__startswith=other.path) & Q(depth__gte=other.depth)
 
@@ -127,6 +135,18 @@ class TreeQuerySet(MP_NodeQuerySet):
 
 
 class PageQuerySet(SearchableQuerySetMixin, TreeQuerySet):
+    def __init__(self, *args, **kwargs):
+        """Set custom instance attributes"""
+        super().__init__(*args, **kwargs)
+        # set by defer_streamfields()
+        self._defer_streamfields = False
+
+    def _clone(self):
+        """Ensure clones inherit custom attribute values."""
+        clone = super()._clone()
+        clone._defer_streamfields = self._defer_streamfields
+        return clone
+
     def live_q(self):
         return Q(live=True)
 
@@ -172,49 +192,50 @@ class PageQuerySet(SearchableQuerySetMixin, TreeQuerySet):
         """
         return self.exclude(self.page_q(other))
 
-    def type_q(self, klass):
-        content_types = ContentType.objects.get_for_models(*[
+    def type_q(self, *types):
+        all_subclasses = set(
             model for model in apps.get_models()
-            if issubclass(model, klass)
-        ]).values()
+            if issubclass(model, types)
+        )
+        content_types = ContentType.objects.get_for_models(*all_subclasses)
+        return Q(content_type__in=list(content_types.values()))
 
-        return Q(content_type__in=list(content_types))
-
-    def type(self, model):
+    def type(self, *types):
         """
         This filters the QuerySet to only contain pages that are an instance
-        of the specified model (including subclasses).
+        of the specified model(s) (including subclasses).
         """
-        return self.filter(self.type_q(model))
+        return self.filter(self.type_q(*types))
 
-    def not_type(self, model):
+    def not_type(self, *types):
         """
-        This filters the QuerySet to not contain any pages which are an instance of the specified model.
+        This filters the QuerySet to exclude any pages which are an instance of the specified model(s).
         """
-        return self.exclude(self.type_q(model))
+        return self.exclude(self.type_q(*types))
 
-    def exact_type_q(self, klass):
-        return Q(content_type=ContentType.objects.get_for_model(klass))
+    def exact_type_q(self, *types):
+        content_types = ContentType.objects.get_for_models(*types)
+        return Q(content_type__in=list(content_types.values()))
 
-    def exact_type(self, model):
+    def exact_type(self, *types):
         """
-        This filters the QuerySet to only contain pages that are an instance of the specified model
+        This filters the QuerySet to only contain pages that are an instance of the specified model(s)
         (matching the model exactly, not subclasses).
         """
-        return self.filter(self.exact_type_q(model))
+        return self.filter(self.exact_type_q(*types))
 
-    def not_exact_type(self, model):
+    def not_exact_type(self, *types):
         """
-        This filters the QuerySet to not contain any pages which are an instance of the specified model
+        This filters the QuerySet to exclude any pages which are an instance of the specified model(s)
         (matching the model exactly, not subclasses).
         """
-        return self.exclude(self.exact_type_q(model))
+        return self.exclude(self.exact_type_q(*types))
 
     def public_q(self):
         from wagtail.core.models import PageViewRestriction
 
         q = Q()
-        for restriction in PageViewRestriction.objects.all():
+        for restriction in PageViewRestriction.objects.select_related('page').all():
             q &= ~self.descendant_of_q(restriction.page, inclusive=True)
         return q
 
@@ -336,15 +357,27 @@ class PageQuerySet(SearchableQuerySetMixin, TreeQuerySet):
         for page in self.live():
             page.unpublish()
 
+    def defer_streamfields(self):
+        """
+        Apply to a queryset to prevent fetching/decoding of StreamField values on
+        evaluation. Useful when working with potentially large numbers of results,
+        where StreamField values are unlikely to be needed. For example, when
+        generating a sitemap or a long list of page links.
+        """
+        clone = self._clone()
+        clone._defer_streamfields = True  # used by specific_iterator()
+        streamfield_names = self.model.get_streamfield_names()
+        if not streamfield_names:
+            return clone
+        return clone.defer(*streamfield_names)
+
     def specific(self, defer=False):
         """
         This efficiently gets all the specific pages for the queryset, using
         the minimum number of queries.
 
-        When the "defer" keyword argument is set to True, only the basic page
-        fields will be loaded and all specific fields will be deferred. It
-        will still generate a query for each page type though (this may be
-        improved to generate only a single query in a future release).
+        When the "defer" keyword argument is set to True, only generic page
+        field values will be loaded and all specific fields will be deferred.
         """
         clone = self._clone()
         if defer:
@@ -359,6 +392,32 @@ class PageQuerySet(SearchableQuerySetMixin, TreeQuerySet):
         """
         return self.descendant_of(site.root_page, inclusive=True)
 
+    def translation_of_q(self, page, inclusive):
+        q = Q(translation_key=page.translation_key)
+
+        if not inclusive:
+            q &= ~Q(pk=page.pk)
+
+        return q
+
+    def translation_of(self, page, inclusive=False):
+        """
+        This filters the QuerySet to only contain pages that are translations of the specified page.
+
+        If inclusive is True, the page itself is returned.
+        """
+        return self.filter(self.translation_of_q(page, inclusive))
+
+    def not_translation_of(self, page, inclusive=False):
+        """
+        This filters the QuerySet to only contain pages that are not translations of the specified page.
+
+        Note, this will include the page itself as the page is technically not a translation of itself.
+        If inclusive is True, we consider the page to be a translation of itself so this excludes the page
+        from the results.
+        """
+        return self.exclude(self.translation_of_q(page, inclusive))
+
 
 def specific_iterator(qs, defer=False):
     """
@@ -367,7 +426,18 @@ def specific_iterator(qs, defer=False):
 
     This should be called from ``PageQuerySet.specific``
     """
-    pks_and_types = qs.values_list('pk', 'content_type')
+    from wagtail.core.models import Page
+
+    annotation_aliases = qs.query.annotations.keys()
+    values = qs.values('pk', 'content_type', *annotation_aliases)
+
+    annotations_by_pk = defaultdict(list)
+    if annotation_aliases:
+        # Extract annotation results keyed by pk so we can reapply to fetched pages.
+        for data in values:
+            annotations_by_pk[data['pk']] = {k: v for k, v in data.items() if k in annotation_aliases}
+
+    pks_and_types = [[v['pk'], v['content_type']] for v in values]
     pks_by_type = defaultdict(list)
     for pk, content_type in pks_and_types:
         pks_by_type[content_type].append(pk)
@@ -378,6 +448,8 @@ def specific_iterator(qs, defer=False):
 
     # Get the specific instances of all pages, one model class at a time.
     pages_by_type = {}
+    missing_pks = []
+
     for content_type, pks in pks_by_type.items():
         # look up model class for this content type, falling back on the original
         # model (i.e. Page) if the more specific one is missing
@@ -386,15 +458,42 @@ def specific_iterator(qs, defer=False):
 
         if defer:
             # Defer all specific fields
-            from wagtail.core.models import Page
             fields = [field.attname for field in Page._meta.get_fields() if field.concrete]
             pages = pages.only(*fields)
+        elif qs._defer_streamfields:
+            pages = pages.defer_streamfields()
 
-        pages_by_type[content_type] = {page.pk: page for page in pages}
+        pages_for_type = {page.pk: page for page in pages}
+        pages_by_type[content_type] = pages_for_type
+        missing_pks.extend(
+            pk for pk in pks if pk not in pages_for_type
+        )
 
-    # Yield all of the pages, in the order they occurred in the original query.
+    # Fetch generic pages to supplement missing items
+    if missing_pks:
+        generic_pages = Page.objects.filter(pk__in=missing_pks).select_related('content_type').in_bulk()
+        warnings.warn(
+            "Specific versions of the following pages could not be found. "
+            "This is most likely because a database migration has removed "
+            "the relevant table or record since the page was created:\n{}".format([
+                {'id': p.id, 'title': p.title, 'type': p.content_type}
+                for p in generic_pages.values()
+            ]), category=RuntimeWarning
+        )
+    else:
+        generic_pages = {}
+
+    # Yield all pages in the order they occurred in the original query.
     for pk, content_type in pks_and_types:
-        yield pages_by_type[content_type][pk]
+        try:
+            page = pages_by_type[content_type][pk]
+        except KeyError:
+            page = generic_pages[pk]
+        if annotation_aliases:
+            # Reapply annotations before returning
+            for annotation, value in annotations_by_pk.get(page.pk, {}).items():
+                setattr(page, annotation, value)
+        yield page
 
 
 class SpecificIterable(BaseIterable):
@@ -402,6 +501,17 @@ class SpecificIterable(BaseIterable):
         return specific_iterator(self.queryset)
 
 
-class DeferredSpecificIterable(BaseIterable):
+class DeferredSpecificIterable(ModelIterable):
     def __iter__(self):
-        return specific_iterator(self.queryset, defer=True)
+        for obj in super().__iter__():
+            if obj.specific_class:
+                yield obj.specific_deferred
+            else:
+                warnings.warn(
+                    "A specific version of the following page could not be returned "
+                    "because the specific page model is not present on the active "
+                    f"branch: <Page id='{obj.id}' title='{obj.title}' "
+                    f"type='{obj.content_type}'>",
+                    category=RuntimeWarning
+                )
+                yield obj
