@@ -10,9 +10,11 @@ urlpatterns = [
     path('edit/<int:document_id>/', documents.edit, name='edit'),
     path('delete/<int:document_id>/', documents.delete, name='delete'),
 
-    path('multiple/add/', multiple.add, name='add_multiple'),
-    path('multiple/<int:doc_id>/', multiple.edit, name='edit_multiple'),
-    path('multiple/<int:doc_id>/delete/', multiple.delete, name='delete_multiple'),
+    path('multiple/add/', multiple.AddView.as_view(), name='add_multiple'),
+    path('multiple/<int:doc_id>/', multiple.EditView.as_view(), name='edit_multiple'),
+    path('multiple/create_from_uploaded_document/<int:uploaded_document_id>/', multiple.CreateFromUploadedDocumentView.as_view(), name='create_multiple_from_uploaded_document'),
+    path('multiple/<int:doc_id>/delete/', multiple.DeleteView.as_view(), name='delete_multiple'),
+    path('multiple/delete_upload/<int:uploaded_document_id>/', multiple.DeleteUploadView.as_view(), name='delete_upload_multiple'),
 
     path('chooser/', chooser.chooser, name='chooser'),
     path('chooser/<int:document_id>/', chooser.document_chosen, name='document_chosen'),
