@@ -13,7 +13,18 @@ class DocumentChooserBlock(ChooserBlock):
     @cached_property
     def widget(self):
         from wagtail.documents.widgets import AdminDocumentChooser
-        return AdminDocumentChooser
+        return AdminDocumentChooser()
+
+    def get_form_state(self, value):
+        value_data = self.widget.get_value_data(value)
+        if value_data is None:
+            return None
+        else:
+            return {
+                'id': value_data['id'],
+                'edit_link': value_data['edit_url'],
+                'title': value_data['title'],
+            }
 
     def render_basic(self, value, context=None):
         if value:
