@@ -1,18 +1,19 @@
 from django.core.paginator import Paginator
-from django.shortcuts import get_object_or_404, render
+from django.shortcuts import get_object_or_404
+from django.template.response import TemplateResponse
 from django.urls import reverse
-from django.utils.translation import ugettext as _
+from django.utils.translation import gettext as _
 
 from wagtail.admin.auth import PermissionPolicyChecker
 from wagtail.admin.forms.search import SearchForm
 from wagtail.admin.modal_workflow import render_modal_workflow
 from wagtail.core import hooks
-from wagtail.core.models import Collection
 from wagtail.documents import get_document_model
 from wagtail.documents.forms import get_document_form
 from wagtail.documents.permissions import permission_policy
 from wagtail.search import index as search_index
 
+
 permission_checker = PermissionPolicyChecker(permission_policy)
 
 
@@ -50,7 +51,9 @@ def chooser(request):
     else:
         uploadform = None
 
-    documents = Document.objects.all()
+    documents = permission_policy.instances_user_has_any_permission_for(
+        request.user, ['choose']
+    )
 
     # allow hooks to modify the queryset
     for hook in hooks.get_hooks('construct_document_chooser_queryset'):
@@ -78,7 +81,7 @@ def chooser(request):
         paginator = Paginator(documents, per_page=10)
         documents = paginator.get_page(request.GET.get('p'))
 
-        return render(request, "wagtaildocs/chooser/results.html", {
+        return TemplateResponse(request, "wagtaildocs/chooser/results.html", {
             'documents': documents,
             'documents_exist': documents_exist,
             'uploadform': uploadform,
@@ -89,11 +92,11 @@ def chooser(request):
     else:
         searchform = SearchForm()
 
-        collections = Collection.objects.all()
+        collections = permission_policy.collections_user_has_permission_for(
+            request.user, 'choose'
+        )
         if len(collections) < 2:
             collections = None
-        else:
-            collections = Collection.order_for_display(collections)
 
         documents = documents.order_by('-created_at')
         documents_exist = documents.exists()
