@@ -17,6 +17,12 @@ from wagtail.utils import sendfile_streaming_backend
 from wagtail.utils.sendfile import sendfile
 
 
+try:
+    from django.utils.http import url_has_allowed_host_and_scheme
+except ImportError:  # fallback for Django 2.2
+    from django.utils.http import is_safe_url as url_has_allowed_host_and_scheme
+
+
 def document_etag(request, document_id, document_filename):
     Document = get_document_model()
     if hasattr(Document, 'file_hash'):
@@ -120,9 +126,13 @@ def authenticate_with_password(request, restriction_id):
     if request.method == 'POST':
         form = PasswordViewRestrictionForm(request.POST, instance=restriction)
         if form.is_valid():
-            restriction.mark_as_passed(request)
+            return_url = form.cleaned_data['return_url']
 
-            return redirect(form.cleaned_data['return_url'])
+            if not url_has_allowed_host_and_scheme(return_url, request.get_host(), request.is_secure()):
+                return_url = settings.LOGIN_REDIRECT_URL
+
+            restriction.mark_as_passed(request)
+            return redirect(return_url)
     else:
         form = PasswordViewRestrictionForm(instance=restriction)
 
