@@ -1,4 +1,5 @@
 from datetime import datetime
+from hashlib import md5
 
 from .exceptions import EmbedUnsupportedProviderException
 from .finders import get_finders
@@ -6,14 +7,17 @@ from .models import Embed
 
 
 def get_embed(url, max_width=None, finder=None):
+    embed_hash = get_embed_hash(url, max_width)
+
     # Check database
     try:
-        return Embed.objects.get(url=url, max_width=max_width)
+        return Embed.objects.get(hash=embed_hash)
     except Embed.DoesNotExist:
         pass
 
     # Get/Call finder
     if not finder:
+
         def finder(url, max_width=None):
             for finder in get_finders():
                 if finder.accept(url):
@@ -38,11 +42,18 @@ def get_embed(url, max_width=None, finder=None):
     if 'html' not in embed_dict or not embed_dict['html']:
         embed_dict['html'] = ''
 
+    # If the finder does not return an thumbnail_url, convert null to '' before inserting into the db
+    if 'thumbnail_url' not in embed_dict or not embed_dict['thumbnail_url']:
+        embed_dict['thumbnail_url'] = ''
+
     # Create database record
     embed, created = Embed.objects.get_or_create(
-        url=url,
-        max_width=max_width,
-        defaults=embed_dict,
+        hash=embed_hash,
+        defaults=dict(
+            url=url,
+            max_width=max_width,
+            **embed_dict,
+        )
     )
 
     # Save
@@ -50,3 +61,12 @@ def get_embed(url, max_width=None, finder=None):
     embed.save()
 
     return embed
+
+
+def get_embed_hash(url, max_width=None):
+    h = md5()
+    h.update(url.encode("utf-8"))
+    if max_width is not None:
+        h.update(b"\n")
+        h.update(str(max_width).encode("utf-8"))
+    return h.hexdigest()
