@@ -13,7 +13,7 @@ from django.urls import reverse
 from wagtail.core import blocks
 from wagtail.embeds import oembed_providers
 from wagtail.embeds.blocks import EmbedBlock, EmbedValue
-from wagtail.embeds.embeds import get_embed
+from wagtail.embeds.embeds import get_embed, get_embed_hash
 from wagtail.embeds.exceptions import EmbedNotFoundException, EmbedUnsupportedProviderException
 from wagtail.embeds.finders import get_finders
 from wagtail.embeds.finders.embedly import AccessDeniedEmbedlyException, EmbedlyException
@@ -128,7 +128,6 @@ class TestEmbeds(TestCase):
         return {
             'title': "Test: " + url,
             'type': 'video',
-            'thumbnail_url': '',
             'width': max_width if max_width else 640,
             'height': 480,
             'html': "<p>Blah blah blah</p>",
@@ -142,6 +141,7 @@ class TestEmbeds(TestCase):
         self.assertEqual(embed.title, "Test: www.test.com/1234")
         self.assertEqual(embed.type, 'video')
         self.assertEqual(embed.width, 400)
+        self.assertEqual(embed.thumbnail_url, '')
 
         # Check ratio calculations
         self.assertEqual(embed.ratio, 480 / 400)
@@ -208,6 +208,14 @@ class TestEmbeds(TestCase):
             get_embed('www.test.com/1234', max_width=400)
 
 
+class TestEmbedHash(TestCase):
+    def test_get_embed_hash(self):
+        url = "www.test.com/1234"
+        self.assertEqual(get_embed_hash(url), "9a4cfc187266026cd68160b5db572629")
+        self.assertEqual(get_embed_hash(url, 0), "946fb9597a6c74ab3cef1699eff7fde7")
+        self.assertEqual(get_embed_hash(url, 1), "427830227a86093b50417e11dbd2f28e")
+
+
 class TestChooser(TestCase, WagtailTestUtils):
     def setUp(self):
         # login
@@ -367,6 +375,14 @@ class TestOembed(TestCase):
             self.assertRaises(EmbedNotFoundException, OEmbedFinder().find_embed,
                               "http://www.youtube.com/watch/")
 
+    @patch('urllib.request.urlopen')
+    def test_oembed_non_json_response(self, urlopen):
+        urlopen.return_value = self.dummy_response
+        self.assertRaises(
+            EmbedNotFoundException, OEmbedFinder().find_embed,
+            "https://www.youtube.com/watch?v=ReblZ7o7lu4"
+        )
+
     @patch('urllib.request.urlopen')
     @patch('json.loads')
     def test_oembed_photo_request(self, loads, urlopen):
@@ -502,7 +518,6 @@ class TestFacebookOEmbed(TestCase):
                     "title": "test_title",
                     "author_name": "test_author",
                     "provider_name": "Facebook",
-                    "thumbnail_url": "test_thumbail_url",
                     "width": "test_width",
                     "height": "test_height",
                     "html": "<blockquote class=\\\"facebook-media\\\">Content</blockquote>"
@@ -523,7 +538,7 @@ class TestFacebookOEmbed(TestCase):
             'title': 'test_title',
             'author_name': 'test_author',
             'provider_name': 'Facebook',
-            'thumbnail_url': 'test_thumbail_url',
+            'thumbnail_url': None,
             'width': 'test_width',
             'height': 'test_height',
             'html': '<blockquote class="facebook-media">Content</blockquote>'
@@ -653,17 +668,6 @@ class TestEmbedBlock(TestCase):
         # Check that get_embed was called correctly
         get_embed.assert_any_call('http://www.example.com/foo')
 
-    def test_render_form(self):
-        """
-        The form field for an EmbedBlock should be a text input containing
-        the URL
-        """
-        block = EmbedBlock()
-
-        form_html = block.render_form(EmbedValue('http://www.example.com/foo'), prefix='myembed')
-        self.assertIn('<input ', form_html)
-        self.assertIn('value="http://www.example.com/foo"', form_html)
-
     def test_value_from_form(self):
         """
         EmbedBlock should be able to turn a URL submitted as part of a form
