@@ -1,6 +1,5 @@
 import json
 
-from django.contrib.auth import get_user_model
 from django.contrib.auth.models import Group, Permission
 from django.core.files.uploadedfile import SimpleUploadedFile
 from django.template.defaultfilters import filesizeformat
@@ -8,14 +7,15 @@ from django.test import TestCase, override_settings
 from django.urls import reverse
 from django.utils.http import RFC3986_SUBDELIMS, urlquote
 
-from wagtail.core.models import Collection, GroupCollectionPermission
+from wagtail.core.models import Collection, GroupCollectionPermission, get_root_collection_id
 from wagtail.images.models import UploadedImage
-from wagtail.images.views.serve import generate_signature
+from wagtail.images.utils import generate_signature
 from wagtail.tests.testapp.models import CustomImage, CustomImageWithAuthor
 from wagtail.tests.utils import WagtailTestUtils
 
 from .utils import Image, get_test_image_file
 
+
 # Get the chars that Django considers safe to leave unescaped in a URL
 urlquote_safechars = RFC3986_SUBDELIMS + str('/~:@')
 
@@ -87,6 +87,14 @@ class TestImageIndexView(TestCase, WagtailTestUtils):
             [collection.name for collection in response.context['collections']],
             ['Root', 'Evil plans', 'Good plans'])
 
+    def test_collection_nesting(self):
+        root_collection = Collection.get_first_root_node()
+        evil_plans = root_collection.add_child(name="Evil plans")
+        evil_plans.add_child(name="Eviler plans")
+
+        response = self.get()
+        # "Eviler Plans" should be prefixed with &#x21b3 (↳) and 4 non-breaking spaces.
+        self.assertContains(response, '&nbsp;&nbsp;&nbsp;&nbsp;&#x21b3 Eviler plans')
 
     def test_tags(self):
         image_two_tags = Image.objects.create(
@@ -107,7 +115,6 @@ class TestImageIndexView(TestCase, WagtailTestUtils):
             or [tag.name for tag in tags] == ["two", "one"]
         )
 
-
     def test_tag_filtering(self):
         Image.objects.create(
             title="Test image with no tags",
@@ -138,7 +145,6 @@ class TestImageIndexView(TestCase, WagtailTestUtils):
         response = self.get({'tag': 'two'})
         self.assertEqual(response.context['images'].paginator.count, 1)
 
-
     def test_tag_filtering_preserves_other_params(self):
         for i in range(1, 100):
             image = Image.objects.create(
@@ -149,7 +155,6 @@ class TestImageIndexView(TestCase, WagtailTestUtils):
                 image.tags.add('even')
                 image.save()
 
-
         response = self.get({'tag': 'even', 'p': 2})
         self.assertEqual(response.status_code, 200)
 
@@ -203,6 +208,15 @@ class TestImageAddView(TestCase, WagtailTestUtils):
         self.assertContains(response, '<label for="id_collection">')
         self.assertContains(response, "Evil plans")
 
+    def test_get_with_collection_nesting(self):
+        root_collection = Collection.get_first_root_node()
+        evil_plans = root_collection.add_child(name="Evil plans")
+        evil_plans.add_child(name="Eviler plans")
+
+        response = self.get()
+        # "Eviler Plans" should be prefixed with &#x21b3 (↳) and 4 non-breaking spaces.
+        self.assertContains(response, '&nbsp;&nbsp;&nbsp;&nbsp;&#x21b3 Eviler plans')
+
     @override_settings(WAGTAILIMAGES_IMAGE_MODEL='tests.CustomImage')
     def test_get_with_custom_image_model(self):
         response = self.get()
@@ -377,14 +391,14 @@ class TestImageAddViewWithLimitedCollectionPermissions(TestCase, WagtailTestUtil
             permission=add_image_permission
         )
 
-        user = get_user_model().objects.create_user(
+        user = self.create_user(
             username='moriarty',
             email='moriarty@example.com',
             password='password'
         )
         user.groups.add(conspirators_group)
 
-        self.client.login(username='moriarty', password='password')
+        self.login(username='moriarty', password='password')
 
     def get(self, params={}):
         return self.client.get(reverse('wagtailimages:add'), params)
@@ -401,6 +415,16 @@ class TestImageAddViewWithLimitedCollectionPermissions(TestCase, WagtailTestUtil
         # is displayed on the form
         self.assertNotContains(response, '<label for="id_collection">')
 
+    def test_get_with_collection_nesting(self):
+        self.evil_plans_collection.add_child(name="Eviler plans")
+
+        response = self.get()
+        self.assertEqual(response.status_code, 200)
+        # Unlike the above test, the user should have access to multiple Collections.
+        self.assertContains(response, '<label for="id_collection">')
+        # "Eviler Plans" should be prefixed with &#x21b3 (↳) and 4 non-breaking spaces.
+        self.assertContains(response, '&nbsp;&nbsp;&nbsp;&nbsp;&#x21b3 Eviler plans')
+
     def test_add(self):
         response = self.post({
             'title': "Test image",
@@ -422,7 +446,7 @@ class TestImageAddViewWithLimitedCollectionPermissions(TestCase, WagtailTestUtil
 
 class TestImageEditView(TestCase, WagtailTestUtils):
     def setUp(self):
-        self.login()
+        self.user = self.login()
 
         # Create an image to edit
         self.image = Image.objects.create(
@@ -454,6 +478,15 @@ class TestImageEditView(TestCase, WagtailTestUtils):
         # definitions are being respected)
         self.assertNotContains(response, 'wagtailadmin/js/draftail.js')
 
+    def test_simple_with_collection_nesting(self):
+        root_collection = Collection.get_first_root_node()
+        evil_plans = root_collection.add_child(name="Evil plans")
+        evil_plans.add_child(name="Eviler plans")
+
+        response = self.get()
+        # "Eviler Plans" should be prefixed with &#x21b3 (↳) and 4 non-breaking spaces.
+        self.assertContains(response, '&nbsp;&nbsp;&nbsp;&nbsp;&#x21b3 Eviler plans')
+
     @override_settings(WAGTAIL_USAGE_COUNT_ENABLED=True)
     def test_with_usage_count(self):
         response = self.get()
@@ -487,6 +520,18 @@ class TestImageEditView(TestCase, WagtailTestUtils):
         self.update_from_db()
         self.assertEqual(self.image.title, "Edited")
 
+    def test_edit_with_limited_permissions(self):
+        self.user.is_superuser = False
+        self.user.user_permissions.add(
+            Permission.objects.get(content_type__app_label='wagtailadmin', codename='access_admin')
+        )
+        self.user.save()
+
+        response = self.post({
+            'title': "Edited",
+        })
+        self.assertEqual(response.status_code, 302)
+
     def test_edit_with_new_image_file(self):
         file_content = get_test_image_file().file.getvalue()
 
@@ -552,7 +597,6 @@ class TestImageEditView(TestCase, WagtailTestUtils):
     def test_get_missing_file_displays_warning_with_custom_storage(self):
         self.check_get_missing_file_displays_warning()
 
-
     def get_content(self, f=None):
         if f is None:
             f = self.image.file
@@ -703,7 +747,7 @@ class TestImageEditViewWithCustomImageModel(TestCase, WagtailTestUtils):
 
 class TestImageDeleteView(TestCase, WagtailTestUtils):
     def setUp(self):
-        self.login()
+        self.user = self.login()
 
         # Create an image to edit
         self.image = Image.objects.create(
@@ -743,6 +787,16 @@ class TestImageDeleteView(TestCase, WagtailTestUtils):
         images = Image.objects.filter(title="Test image")
         self.assertEqual(images.count(), 0)
 
+    def test_delete_with_limited_permissions(self):
+        self.user.is_superuser = False
+        self.user.user_permissions.add(
+            Permission.objects.get(content_type__app_label='wagtailadmin', codename='access_admin')
+        )
+        self.user.save()
+
+        response = self.post()
+        self.assertEqual(response.status_code, 302)
+
 
 class TestImageChooserView(TestCase, WagtailTestUtils):
     def setUp(self):
@@ -761,6 +815,91 @@ class TestImageChooserView(TestCase, WagtailTestUtils):
         # draftail should NOT be a standard JS include on this page
         self.assertNotIn('wagtailadmin/js/draftail.js', response_json['html'])
 
+    def test_simple_with_collection_nesting(self):
+        root_collection = Collection.get_first_root_node()
+        evil_plans = root_collection.add_child(name="Evil plans")
+        evil_plans.add_child(name="Eviler plans")
+
+        response = self.get()
+        # "Eviler Plans" should be prefixed with &#x21b3 (↳) and 4 non-breaking spaces.
+        self.assertContains(response, '&nbsp;&nbsp;&nbsp;&nbsp;&#x21b3 Eviler plans')
+
+    def test_choose_permissions(self):
+        # Create group with access to admin and Chooser permission on one Collection, but not another.
+        bakers_group = Group.objects.create(name='Bakers')
+        access_admin_perm = Permission.objects.get(
+            content_type__app_label='wagtailadmin',
+            codename='access_admin'
+        )
+        bakers_group.permissions.add(access_admin_perm)
+        # Create the "Bakery" Collection and grant "choose" permission to the Bakers group.
+        root = Collection.objects.get(id=get_root_collection_id())
+        bakery_collection = root.add_child(instance=Collection(name='Bakery'))
+        GroupCollectionPermission.objects.create(
+            group=bakers_group,
+            collection=bakery_collection,
+            permission=Permission.objects.get(
+                content_type__app_label='wagtailimages',
+                codename='choose_image'
+            )
+        )
+        # Create the "Office" Collection and _don't_ grant any permissions to the Bakers group.
+        office_collection = root.add_child(instance=Collection(name='Office'))
+
+        # Create a new user in the Bakers group, and log in as them.
+        # Can't use self.user because it's a superuser.
+        baker = self.create_user(username='baker', password='password')
+        baker.groups.add(bakers_group)
+        self.login(username='baker', password='password')
+
+        # Add an image to each Collection.
+        sweet_buns = Image.objects.create(
+            title='SweetBuns.jpg',
+            file=get_test_image_file(),
+            collection=bakery_collection,
+        )
+        poster = Image.objects.create(
+            title='PromotionalPoster.jpg',
+            file=get_test_image_file(),
+            collection=office_collection,
+        )
+
+        # Open the image chooser
+        response = self.get()
+        self.assertEqual(response.status_code, 200)
+        self.assertTemplateUsed(response, 'wagtailimages/chooser/chooser.html')
+
+        # Confirm that the Baker can see the sweet buns, but not the promotional poster.
+        self.assertContains(response, sweet_buns.title)
+        self.assertNotContains(response, poster.title)
+
+        # Confirm that the Collection chooser is not visible, because the Baker cannot
+        # choose from multiple Collections.
+        self.assertNotContains(response, 'Collection:')
+
+        # We now let the Baker choose from the Office collection.
+        GroupCollectionPermission.objects.create(
+            group=Group.objects.get(name='Bakers'),
+            collection=Collection.objects.get(name='Office'),
+            permission=Permission.objects.get(
+                content_type__app_label='wagtailimages',
+                codename='choose_image'
+            )
+        )
+
+        # Open the image chooser again.
+        response = self.get()
+        self.assertEqual(response.status_code, 200)
+        self.assertTemplateUsed(response, 'wagtailimages/chooser/chooser.html')
+
+        # Confirm that the Baker can now see both images.
+        self.assertContains(response, sweet_buns.title)
+        self.assertContains(response, poster.title)
+
+        # Ensure that the Collection chooser IS visible, because the Baker can now
+        # choose from multiple Collections.
+        self.assertContains(response, 'Collection:')
+
     @override_settings(WAGTAILIMAGES_IMAGE_MODEL='tests.CustomImage')
     def test_with_custom_image_model(self):
         response = self.get()
@@ -895,9 +1034,19 @@ class TestImageChooserSelectFormatView(TestCase, WagtailTestUtils):
         response = self.get(params={'alt_text': "some previous alt text"})
         self.assertEqual(response.status_code, 200)
         self.assertContains(response, 'value=\\"some previous alt text\\"')
+        self.assertNotContains(response, 'id=\\"id_image-chooser-insertion-image_is_decorative\\" checked')
+
+    def test_with_edit_params_no_alt_text_marks_as_decorative(self):
+        response = self.get(params={'alt_text': ""})
+        self.assertEqual(response.status_code, 200)
+        self.assertContains(response, 'id=\\"id_image-chooser-insertion-image_is_decorative\\" checked')
 
     def test_post_response(self):
-        response = self.post({'image-chooser-insertion-format': 'left', 'image-chooser-insertion-alt_text': 'Arthur "two sheds" Jackson'})
+        response = self.post({
+            'image-chooser-insertion-format': 'left',
+            'image-chooser-insertion-image_is_decorative': False,
+            'image-chooser-insertion-alt_text': 'Arthur "two sheds" Jackson',
+        })
 
         self.assertEqual(response.status_code, 200)
         self.assertEqual(response['Content-Type'], 'application/json')
@@ -912,6 +1061,27 @@ class TestImageChooserSelectFormatView(TestCase, WagtailTestUtils):
         self.assertEqual(result['alt'], 'Arthur "two sheds" Jackson')
         self.assertIn('alt="Arthur &quot;two sheds&quot; Jackson"', result['html'])
 
+    def test_post_response_image_is_decorative_discards_alt_text(self):
+        response = self.post({
+            'image-chooser-insertion-format': 'left',
+            'image-chooser-insertion-alt_text': 'Arthur "two sheds" Jackson',
+            'image-chooser-insertion-image_is_decorative': True,
+        })
+        response_json = json.loads(response.content.decode())
+        result = response_json['result']
+
+        self.assertEqual(result['alt'], '')
+        self.assertIn('alt=""', result['html'])
+
+    def test_post_response_image_is_not_decorative_missing_alt_text(self):
+        response = self.post({
+            'image-chooser-insertion-format': 'left',
+            'image-chooser-insertion-alt_text': '',
+            'image-chooser-insertion-image_is_decorative': False,
+        })
+        response_json = json.loads(response.content.decode())
+        self.assertIn('Please add some alt text for your image or mark it as decorative', response_json['html'])
+
 
 class TestImageChooserUploadView(TestCase, WagtailTestUtils):
     def setUp(self):
@@ -1072,14 +1242,14 @@ class TestImageChooserUploadViewWithLimitedPermissions(TestCase, WagtailTestUtil
             permission=add_image_permission
         )
 
-        user = get_user_model().objects.create_user(
+        user = self.create_user(
             username='moriarty',
             email='moriarty@example.com',
             password='password'
         )
         user.groups.add(conspirators_group)
 
-        self.client.login(username='moriarty', password='password')
+        self.login(username='moriarty', password='password')
 
     def test_get(self):
         response = self.client.get(reverse('wagtailimages:chooser_upload'))
@@ -1170,7 +1340,7 @@ class TestMultipleImageUploader(TestCase, WagtailTestUtils):
         # Check response
         self.assertEqual(response.status_code, 200)
         self.assertEqual(response['Content-Type'], 'application/json')
-        self.assertTemplateUsed(response, 'wagtailimages/multiple/edit_form.html')
+        self.assertTemplateUsed(response, 'wagtailadmin/generic/multiple_upload/edit_form.html')
 
         # Check image
         self.assertIn('image', response.context)
@@ -1318,7 +1488,7 @@ class TestMultipleImageUploader(TestCase, WagtailTestUtils):
         # Check response
         self.assertEqual(response.status_code, 200)
         self.assertEqual(response['Content-Type'], 'application/json')
-        self.assertTemplateUsed(response, 'wagtailimages/multiple/edit_form.html')
+        self.assertTemplateUsed(response, 'wagtailadmin/generic/multiple_upload/edit_form.html')
 
         # Check that a form error was raised
         self.assertFormError(response, 'form', 'title', "This field is required.")
@@ -1415,7 +1585,7 @@ class TestMultipleImageUploaderWithCustomImageModel(TestCase, WagtailTestUtils):
         # Check response
         self.assertEqual(response.status_code, 200)
         self.assertEqual(response['Content-Type'], 'application/json')
-        self.assertTemplateUsed(response, 'wagtailimages/multiple/edit_form.html')
+        self.assertTemplateUsed(response, 'wagtailadmin/generic/multiple_upload/edit_form.html')
 
         # Check image
         self.assertIn('image', response.context)
@@ -1488,7 +1658,7 @@ class TestMultipleImageUploaderWithCustomImageModel(TestCase, WagtailTestUtils):
         # Check response
         self.assertEqual(response.status_code, 200)
         self.assertEqual(response['Content-Type'], 'application/json')
-        self.assertTemplateUsed(response, 'wagtailimages/multiple/edit_form.html')
+        self.assertTemplateUsed(response, 'wagtailadmin/generic/multiple_upload/edit_form.html')
 
     def test_edit_post(self):
         """
@@ -1543,7 +1713,7 @@ class TestMultipleImageUploaderWithCustomImageModel(TestCase, WagtailTestUtils):
         # Check response
         self.assertEqual(response.status_code, 200)
         self.assertEqual(response['Content-Type'], 'application/json')
-        self.assertTemplateUsed(response, 'wagtailimages/multiple/edit_form.html')
+        self.assertTemplateUsed(response, 'wagtailadmin/generic/multiple_upload/edit_form.html')
 
         response_json = json.loads(response.content.decode())
         # Check JSON
@@ -1628,7 +1798,7 @@ class TestMultipleImageUploaderWithCustomRequiredFields(TestCase, WagtailTestUti
         # Check response
         self.assertEqual(response.status_code, 200)
         self.assertEqual(response['Content-Type'], 'application/json')
-        self.assertTemplateUsed(response, 'wagtailimages/multiple/edit_form.html')
+        self.assertTemplateUsed(response, 'wagtailadmin/generic/multiple_upload/edit_form.html')
 
         # Check image
         self.assertIn('uploaded_image', response.context)
@@ -1763,7 +1933,7 @@ class TestMultipleImageUploaderWithCustomRequiredFields(TestCase, WagtailTestUti
         self.assertEqual(response['Content-Type'], 'application/json')
 
         # Make sure the image is deleted
-        self.assertFalse(CustomImageWithAuthor.objects.filter(id=self.uploaded_image.id).exists())
+        self.assertFalse(UploadedImage.objects.filter(id=self.uploaded_image.id).exists())
 
         # Check JSON
         response_json = json.loads(response.content.decode())
@@ -1950,7 +2120,7 @@ class TestEditOnlyPermissions(TestCase, WagtailTestUtils):
         )
 
         # Create a user with change_image permission but not add_image
-        user = get_user_model().objects.create_user(
+        user = self.create_user(
             username='changeonly', email='changeonly@example.com', password='password'
         )
         change_permission = Permission.objects.get(content_type__app_label='wagtailimages', codename='change_image')
@@ -1965,7 +2135,7 @@ class TestEditOnlyPermissions(TestCase, WagtailTestUtils):
         )
 
         user.groups.add(image_changers_group)
-        self.assertTrue(self.client.login(username='changeonly', password='password'))
+        self.login(username='changeonly', password='password')
 
     def test_get_index(self):
         response = self.client.get(reverse('wagtailimages:index'))
@@ -2012,7 +2182,7 @@ class TestImageAddMultipleView(TestCase, WagtailTestUtils):
         self.assertTemplateUsed(response, 'wagtailimages/multiple/add.html')
 
     def test_as_ordinary_editor(self):
-        user = get_user_model().objects.create_user(username='editor', email='editor@email.com', password='password')
+        user = self.create_user(username='editor', password='password')
 
         add_permission = Permission.objects.get(content_type__app_label='wagtailimages', codename='add_image')
         admin_permission = Permission.objects.get(content_type__app_label='wagtailadmin', codename='access_admin')
@@ -2021,7 +2191,7 @@ class TestImageAddMultipleView(TestCase, WagtailTestUtils):
         GroupCollectionPermission.objects.create(group=image_adders_group, collection=Collection.get_first_root_node(), permission=add_permission)
         user.groups.add(image_adders_group)
 
-        self.client.login(username='editor', password='password')
+        self.login(username='editor', password='password')
 
         response = self.client.get(reverse('wagtailimages:add_multiple'))
         self.assertEqual(response.status_code, 200)
