@@ -1,10 +1,11 @@
-from django.conf.urls import include, url
-from django.urls import reverse
+from django.urls import include, path, reverse
 from django.utils.html import format_html
-from django.utils.translation import ugettext_lazy as _
-from django.utils.translation import ugettext, ungettext
+from django.utils.translation import gettext
+from django.utils.translation import gettext_lazy as _
+from django.utils.translation import ngettext
 
 import wagtail.admin.rich_text.editors.draftail.features as draftail_features
+
 from wagtail.admin.menu import MenuItem
 from wagtail.admin.navigation import get_site_for_user
 from wagtail.admin.rich_text import HalloPlugin
@@ -23,7 +24,7 @@ from wagtail.images.rich_text.editor_html import EditorHTMLImageConversionRule
 @hooks.register('register_admin_urls')
 def register_admin_urls():
     return [
-        url(r'^images/', include(admin_urls, namespace='wagtailimages')),
+        path('images/', include(admin_urls, namespace='wagtailimages')),
     ]
 
 
@@ -43,7 +44,7 @@ class ImagesMenuItem(MenuItem):
 def register_images_menu_item():
     return ImagesMenuItem(
         _('Images'), reverse('wagtailimages:index'),
-        name='images', classnames='icon icon-image', order=300
+        name='images', icon_name='image', order=300
     )
 
 
@@ -85,7 +86,7 @@ def register_image_feature(features):
         'draftail', 'image', draftail_features.EntityFeature({
             'type': 'IMAGE',
             'icon': 'image',
-            'description': ugettext('Image'),
+            'description': gettext('Image'),
             # We do not want users to be able to copy-paste hotlinked images into rich text.
             # Keep only the attributes Wagtail needs.
             'attributes': ['id', 'src', 'alt', 'format'],
@@ -117,6 +118,7 @@ def register_image_operations():
         ('height', image_operations.WidthHeightOperation),
         ('scale', image_operations.ScaleOperation),
         ('jpegquality', image_operations.JPEGQualityOperation),
+        ('webpquality', image_operations.WebPQualityOperation),
         ('format', image_operations.FormatOperation),
         ('bgcolor', image_operations.BackgroundColorOperation),
     ]
@@ -157,7 +159,7 @@ def register_images_search_area():
     return ImagesSearchArea(
         _('Images'), reverse('wagtailimages:index'),
         name='images',
-        classnames='icon icon-image',
+        icon_name='image',
         order=200)
 
 
@@ -173,7 +175,7 @@ def describe_collection_docs(collection):
         url = reverse('wagtailimages:index') + ('?collection_id=%d' % collection.id)
         return {
             'count': images_count,
-            'count_text': ungettext(
+            'count_text': ngettext(
                 "%(count)s image",
                 "%(count)s images",
                 images_count
