@@ -333,6 +333,17 @@ class TestElasticsearch5SearchQuery(TestCase):
         expected_result = {'match_phrase': {'title': "Hello world"}}
         self.assertDictEqual(query_compiler.get_inner_query(), expected_result)
 
+    def test_year_filter(self):
+        # Create a query
+        query_compiler = self.query_compiler_class(models.Book.objects.filter(publication_date__year__gt=1900), "Hello")
+
+        # Check it
+        expected_result = {'bool': {'filter': [
+            {'match': {'content_type': 'searchtests.Book'}},
+            {'range': {'publication_date_filter': {'gt': 1900}}}
+        ], 'must': {'multi_match': {'query': 'Hello', 'fields': ['_all', '_partials']}}}}
+        self.assertDictEqual(query_compiler.get_query(), expected_result)
+
 
 class TestElasticsearch5SearchResults(TestCase):
     fixtures = ['search']
@@ -743,7 +754,7 @@ class TestElasticsearch5MappingInheritance(TestCase):
         self.assertDictEqual(document, expected_result)
 
 
-@mock.patch('wagtail.search.backends.elasticsearch2.Elasticsearch')
+@mock.patch('wagtail.search.backends.elasticsearch5.Elasticsearch')
 class TestBackendConfiguration(TestCase):
     def test_default_settings(self, Elasticsearch):
         Elasticsearch5SearchBackend(params={})
