@@ -17,13 +17,15 @@ from taggit.models import Tag
 from wagtail.admin.edit_handlers import FieldPanel
 from wagtail.admin.forms import WagtailAdminModelForm
 from wagtail.core import hooks
+from wagtail.core.blocks.field_block import FieldBlockAdapter
 from wagtail.core.models import Locale, Page
 from wagtail.snippets.action_menu import ActionMenuItem, get_base_snippet_action_menu_items
 from wagtail.snippets.blocks import SnippetChooserBlock
 from wagtail.snippets.edit_handlers import SnippetChooserPanel
 from wagtail.snippets.models import SNIPPET_MODELS, register_snippet
 from wagtail.snippets.views.snippets import get_snippet_edit_handler
-from wagtail.snippets.widgets import SnippetListingButton
+from wagtail.snippets.widgets import (
+    AdminSnippetChooser, SnippetChooserAdapter, SnippetListingButton)
 from wagtail.tests.snippets.forms import FancySnippetForm
 from wagtail.tests.snippets.models import (
     AlphaSnippet, FancySnippet, FileUploadSnippet, RegisterDecorator, RegisterFunction,
@@ -287,9 +289,9 @@ class TestSnippetCreateView(TestCase, WagtailTestUtils):
         response = self.get()
         self.assertEqual(response.status_code, 200)
         self.assertTemplateUsed(response, 'wagtailsnippets/snippets/create.html')
-        self.assertNotContains(response, '<ul class="tab-nav merged" role="tablist">')
-        self.assertNotContains(response, '<a href="#tab-advert" class="active">Advert</a>', html=True)
-        self.assertNotContains(response, '<a href="#tab-other" class="">Other</a>', html=True)
+        self.assertNotContains(response, '<ul data-tab-nav role="tablist" data-current-tab="advert">')
+        self.assertNotContains(response, '<a href="#tab-advert" class="active" data-tab="advert">Advert</a>', html=True)
+        self.assertNotContains(response, '<a href="#tab-other" class="" data-tab="other">Other</a>', html=True)
 
     def test_snippet_with_tabbed_interface(self):
         response = self.client.get(reverse('wagtailsnippets:add',
@@ -297,9 +299,9 @@ class TestSnippetCreateView(TestCase, WagtailTestUtils):
 
         self.assertEqual(response.status_code, 200)
         self.assertTemplateUsed(response, 'wagtailsnippets/snippets/create.html')
-        self.assertContains(response, '<ul class="tab-nav merged" role="tablist">')
-        self.assertContains(response, '<a href="#tab-advert" class="active">Advert</a>', html=True)
-        self.assertContains(response, '<a href="#tab-other" class="">Other</a>', html=True)
+        self.assertContains(response, '<ul data-tab-nav role="tablist" data-current-tab="advert">')
+        self.assertContains(response, '<a href="#tab-advert" class="active" data-tab="advert">Advert</a>', html=True)
+        self.assertContains(response, '<a href="#tab-other" class="" data-tab="other">Other</a>', html=True)
 
     def test_create_with_limited_permissions(self):
         self.user.is_superuser = False
@@ -429,6 +431,18 @@ class TestSnippetCreateView(TestCase, WagtailTestUtils):
 
         self.assertContains(response, '<button type="submit" name="test" value="Test" class="button action-secondary"><svg class="icon icon-undo icon" aria-hidden="true" focusable="false"><use href="#icon-undo"></use></svg>Test</button>', html=True)
 
+    def test_register_snippet_action_menu_item_as_none(self):
+        def hook_func(model):
+            return None
+
+        with self.register_hook('register_snippet_action_menu_item', hook_func):
+            get_base_snippet_action_menu_items.cache_clear()
+
+            response = self.get()
+
+        get_base_snippet_action_menu_items.cache_clear()
+        self.assertEqual(response.status_code, 200)
+
     def test_construct_snippet_action_menu(self):
         class TestSnippetActionMenuItem(ActionMenuItem):
             label = "Test"
@@ -454,7 +468,7 @@ class TestSnippetCreateView(TestCase, WagtailTestUtils):
             response = self.get()
 
         self.assertContains(response, '<button type="submit" name="test" value="Test" class="button action-secondary"><svg class="icon icon-undo icon" aria-hidden="true" focusable="false"><use href="#icon-undo"></use></svg>Test</button>', html=True)
-        self.assertNotContains(response, 'Save')
+        self.assertNotContains(response, "<em>'Save'</em>")
 
 
 @override_settings(WAGTAIL_I18N_ENABLED=True)
@@ -528,9 +542,9 @@ class TestSnippetEditView(BaseTestSnippetEditView):
         response = self.get()
         self.assertEqual(response.status_code, 200)
         self.assertTemplateUsed(response, 'wagtailsnippets/snippets/edit.html')
-        self.assertNotContains(response, '<ul class="tab-nav merged" role="tablist">')
-        self.assertNotContains(response, '<a href="#advert" class="active">Advert</a>', html=True)
-        self.assertNotContains(response, '<a href="#other" class="">Other</a>', html=True)
+        self.assertNotContains(response, '<ul data-tab-nav role="tablist" data-current-tab="advert">')
+        self.assertNotContains(response, '<a href="#advert" class="active" data-tab="advert">Advert</a>', html=True)
+        self.assertNotContains(response, '<a href="#other" class="" data-tab="other">Other</a>', html=True)
 
     def test_non_existant_model(self):
         response = self.client.get(reverse('wagtailsnippets:edit', args=('tests', 'foo', quote(self.test_snippet.pk))))
@@ -670,7 +684,7 @@ class TestSnippetEditView(BaseTestSnippetEditView):
         with self.register_hook('construct_snippet_action_menu', hook_func):
             response = self.get()
 
-        self.assertNotContains(response, 'Save')
+        self.assertNotContains(response, '<em>Save</em>')
 
 
 class TestEditTabbedSnippet(BaseTestSnippetEditView):
@@ -687,9 +701,9 @@ class TestEditTabbedSnippet(BaseTestSnippetEditView):
 
         self.assertEqual(response.status_code, 200)
         self.assertTemplateUsed(response, 'wagtailsnippets/snippets/edit.html')
-        self.assertContains(response, '<ul class="tab-nav merged" role="tablist">')
-        self.assertContains(response, '<a href="#tab-advert" class="active">Advert</a>', html=True)
-        self.assertContains(response, '<a href="#tab-other" class="">Other</a>', html=True)
+        self.assertContains(response, '<ul data-tab-nav role="tablist" data-current-tab="advert">')
+        self.assertContains(response, '<a href="#tab-advert" class="active" data-tab="advert">Advert</a>', html=True)
+        self.assertContains(response, '<a href="#tab-other" class="" data-tab="other">Other</a>', html=True)
 
 
 class TestEditFileUploadSnippet(BaseTestSnippetEditView):
@@ -1344,18 +1358,24 @@ class TestSnippetChooserBlock(TestCase):
         test_advert = Advert.objects.get(text='test_advert')
         self.assertEqual(block.to_python(test_advert.id), test_advert)
 
-    def test_form_render(self):
+    def test_adapt(self):
         block = SnippetChooserBlock(Advert, help_text="pick an advert, any advert")
 
-        empty_form_html = block.render_form(None, 'advert')
-        self.assertInHTML('<input id="advert" name="advert" placeholder="" type="hidden" />', empty_form_html)
-        self.assertIn('createSnippetChooser("advert", "tests/advert");', empty_form_html)
+        block.set_name('test_snippetchooserblock')
+        js_args = FieldBlockAdapter().js_args(block)
 
-        test_advert = Advert.objects.get(text='test_advert')
-        test_advert_form_html = block.render_form(test_advert, 'advert')
-        expected_html = '<input id="advert" name="advert" placeholder="" type="hidden" value="%d" />' % test_advert.id
-        self.assertInHTML(expected_html, test_advert_form_html)
-        self.assertIn("pick an advert, any advert", test_advert_form_html)
+        self.assertEqual(js_args[0], 'test_snippetchooserblock')
+        self.assertIsInstance(js_args[1], AdminSnippetChooser)
+        self.assertEqual(js_args[1].target_model, Advert)
+        self.assertEqual(js_args[2], {
+            'label': 'Test snippetchooserblock',
+            'required': True,
+            'icon': 'snippet',
+            'helpText': 'pick an advert, any advert',
+            'classname': 'field model_choice_field widget-admin_snippet_chooser fieldname-test_snippetchooserblock',
+            'showAddCommentButton': True,
+            'strings': {'ADD_COMMENT': 'Add Comment'}
+        })
 
     def test_form_response(self):
         block = SnippetChooserBlock(Advert)
@@ -1380,6 +1400,18 @@ class TestSnippetChooserBlock(TestCase):
         self.assertEqual(nonrequired_block.clean(None), None)
 
 
+class TestAdminSnippetChooserWidget(TestCase, WagtailTestUtils):
+    def test_adapt(self):
+        widget = AdminSnippetChooser(Advert)
+
+        js_args = SnippetChooserAdapter().js_args(widget)
+
+        self.assertInHTML('<input type="hidden" name="__NAME__" id="__ID__">', js_args[0])
+        self.assertIn('>Choose advert<', js_args[0])
+        self.assertEqual(js_args[1], '__ID__')
+        self.assertEqual(js_args[2], 'tests/advert')
+
+
 class TestSnippetListViewWithCustomPrimaryKey(TestCase, WagtailTestUtils):
     def setUp(self):
         self.login()
@@ -1491,18 +1523,24 @@ class TestSnippetChooserBlockWithCustomPrimaryKey(TestCase):
         # None should deserialize to None
         self.assertEqual(block.to_python(None), None)
 
-    def test_form_render(self):
+    def test_adapt(self):
         block = SnippetChooserBlock(AdvertWithCustomPrimaryKey, help_text="pick an advert, any advert")
 
-        empty_form_html = block.render_form(None, 'advertwithcustomprimarykey')
-        self.assertInHTML('<input id="advertwithcustomprimarykey" name="advertwithcustomprimarykey" placeholder="" type="hidden" />', empty_form_html)
-        self.assertIn('createSnippetChooser("advertwithcustomprimarykey", "tests/advertwithcustomprimarykey");', empty_form_html)
+        block.set_name('test_snippetchooserblock')
+        js_args = FieldBlockAdapter().js_args(block)
 
-        test_advert = AdvertWithCustomPrimaryKey.objects.get(pk='advert/01')
-        test_advert_form_html = block.render_form(test_advert, 'advertwithcustomprimarykey')
-        expected_html = '<input id="advertwithcustomprimarykey" name="advertwithcustomprimarykey" placeholder="" type="hidden" value="%s" />' % test_advert.pk
-        self.assertInHTML(expected_html, test_advert_form_html)
-        self.assertIn("pick an advert, any advert", test_advert_form_html)
+        self.assertEqual(js_args[0], 'test_snippetchooserblock')
+        self.assertIsInstance(js_args[1], AdminSnippetChooser)
+        self.assertEqual(js_args[1].target_model, AdvertWithCustomPrimaryKey)
+        self.assertEqual(js_args[2], {
+            'label': 'Test snippetchooserblock',
+            'required': True,
+            'icon': 'snippet',
+            'helpText': 'pick an advert, any advert',
+            'classname': 'field model_choice_field widget-admin_snippet_chooser fieldname-test_snippetchooserblock',
+            'showAddCommentButton': True,
+            'strings': {'ADD_COMMENT': 'Add Comment'}
+        })
 
     def test_form_response(self):
         block = SnippetChooserBlock(AdvertWithCustomPrimaryKey)
