@@ -1,16 +1,17 @@
 """An alternative urlconf module where Wagtail front-end URLs
 are rooted at '/site/' rather than '/'"""
 
-from django.conf.urls import include, url
+from django.urls import include, path
 
 from wagtail.admin import urls as wagtailadmin_urls
 from wagtail.core import urls as wagtail_urls
 from wagtail.documents import urls as wagtaildocs_urls
 from wagtail.images import urls as wagtailimages_urls
 
+
 urlpatterns = [
-    url(r'^admin/', include(wagtailadmin_urls)),
-    url(r'^documents/', include(wagtaildocs_urls)),
-    url(r'^images/', include(wagtailimages_urls)),
-    url(r'^site/', include(wagtail_urls)),
+    path('admin/', include(wagtailadmin_urls)),
+    path('documents/', include(wagtaildocs_urls)),
+    path('images/', include(wagtailimages_urls)),
+    path('site/', include(wagtail_urls)),
 ]
