@@ -1,4 +1,5 @@
 import warnings
+
 from itertools import groupby
 from operator import itemgetter
 
@@ -14,16 +15,18 @@ from django.db import transaction
 from django.db.models.fields import BLANK_CHOICE_DASH
 from django.template.loader import render_to_string
 from django.utils.html import mark_safe
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import get_language_info
+from django.utils.translation import gettext_lazy as _
 
 from wagtail.admin.localization import get_available_admin_languages, get_available_admin_time_zones
-from wagtail.admin.widgets import AdminPageChooser
+from wagtail.admin.widgets import AdminPageChooser, SwitchInput
 from wagtail.core import hooks
 from wagtail.core.models import (
     PAGE_PERMISSION_TYPE_CHOICES, PAGE_PERMISSION_TYPES, GroupPagePermission, Page,
     UserPagePermissionsProxy)
 from wagtail.users.models import UserProfile
 
+
 User = get_user_model()
 
 # The standard fields each user model is expected to have, as a minimum.
@@ -83,11 +86,11 @@ class UserForm(UsernameForm):
 
     password1 = forms.CharField(
         label=_('Password'), required=False,
-        widget=forms.PasswordInput,
+        widget=forms.PasswordInput(attrs={'autocomplete': 'new-password'}),
         help_text=_("Leave blank if not changing."))
     password2 = forms.CharField(
         label=_("Password confirmation"), required=False,
-        widget=forms.PasswordInput,
+        widget=forms.PasswordInput(attrs={'autocomplete': 'new-password'}),
         help_text=_("Enter the same password as above, for verification."))
 
     is_superuser = forms.BooleanField(
@@ -380,45 +383,28 @@ class NotificationPreferencesForm(forms.ModelForm):
         if not user_perms.can_edit_pages():
             del self.fields['approved_notifications']
             del self.fields['rejected_notifications']
+            del self.fields['updated_comments_notifications']
 
     class Meta:
         model = UserProfile
-        fields = ("submitted_notifications", "approved_notifications", "rejected_notifications")
+        fields = ['submitted_notifications', 'approved_notifications', 'rejected_notifications', 'updated_comments_notifications']
+        widgets = {
+            'submitted_notifications': SwitchInput(),
+            'approved_notifications': SwitchInput(),
+            'rejected_notifications': SwitchInput(),
+            'updated_comments_notifications': SwitchInput(),
+        }
 
 
 def _get_language_choices():
-    return sorted(BLANK_CHOICE_DASH + get_available_admin_languages(),
+    language_choices = [
+        (lang_code, get_language_info(lang_code)['name_local'])
+        for lang_code, lang_name in get_available_admin_languages()
+    ]
+    return sorted(BLANK_CHOICE_DASH + language_choices,
                   key=lambda l: l[1].lower())
 
 
-class PreferredLanguageForm(forms.ModelForm):
-    preferred_language = forms.ChoiceField(
-        required=False, choices=_get_language_choices,
-        label=_('Preferred language')
-    )
-
-    class Meta:
-        model = UserProfile
-        fields = ("preferred_language",)
-
-
-class EmailForm(forms.ModelForm):
-    email = forms.EmailField(required=True, label=_('Email'))
-
-    class Meta:
-        model = User
-        fields = ("email",)
-
-
-class NameForm(forms.ModelForm):
-    first_name = forms.CharField(required=True, label=_('First Name'))
-    last_name = forms.CharField(required=True, label=_('Last Name'))
-
-    class Meta:
-        model = User
-        fields = ("first_name", "last_name",)
-
-
 def _get_time_zone_choices():
     time_zones = [(tz, str(l18n.tz_fullnames.get(tz, tz)))
                   for tz in get_available_admin_time_zones()]
@@ -426,20 +412,51 @@ def _get_time_zone_choices():
     return BLANK_CHOICE_DASH + time_zones
 
 
-class CurrentTimeZoneForm(forms.ModelForm):
+class LocalePreferencesForm(forms.ModelForm):
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+
+        if len(get_available_admin_languages()) <= 1:
+            del self.fields['preferred_language']
+
+        if len(get_available_admin_time_zones()) <= 1:
+            del self.fields['current_time_zone']
+
+    preferred_language = forms.ChoiceField(
+        required=False, choices=_get_language_choices,
+        label=_("Preferred language")
+    )
+
     current_time_zone = forms.ChoiceField(
         required=False, choices=_get_time_zone_choices,
-        label=_('Current time zone')
+        label=_("Current time zone")
     )
 
     class Meta:
         model = UserProfile
-        fields = ("current_time_zone",)
+        fields = ['preferred_language', 'current_time_zone']
+
+
+class NameEmailForm(forms.ModelForm):
+    first_name = forms.CharField(required=True, label=_('First Name'))
+    last_name = forms.CharField(required=True, label=_('Last Name'))
+    email = forms.EmailField(required=True, label=_('Email'))
+
+    def __init__(self, *args, **kwargs):
+        from wagtail.admin.views.account import email_management_enabled
+        super().__init__(*args, **kwargs)
+
+        if not email_management_enabled():
+            del self.fields['email']
+
+    class Meta:
+        model = User
+        fields = ['first_name', 'last_name', 'email']
 
 
 class AvatarPreferencesForm(forms.ModelForm):
     avatar = forms.ImageField(
-        label=_("Upload a profile picture"), required=True
+        label=_("Upload a profile picture"), required=False
     )
 
     def __init__(self, *args, **kwargs):
